package web.admin.views

import ant.cps.{Breadcrumb, Card, Col, Row}
import ant.cps.Breadcrumb.Item
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import monix.execution.cancelables.MultiAssignmentCancelable
import monix.reactive.subjects.PublishSubject
import org.scalajs.dom
import relay.components.FragmentContainer
import relay.graphql.TaggedNode
import relay.components._
import vdom.html_<^._
import web.admin.cps.AdminCore
import web.admin.data.BaseViewerFrag
import web.admin.routes._
import web.cps.chat.{ChatGrammar, ChatInterface}
import web.cps.{BreadcrumbDisplay, CardThick, Sectional}
import web.react.RelayBackend

import scala.scalajs.js
import scala.scalajs.js.{|, JSON, UndefOr}

object ClaimView {

  case class Props(rc: RouterCtl[Page], page: ClaimPage)

  @gql("""
    query ClaimView($claimId: String!, $first: Int!) @scalajs(useNulls: true) {
      viewer {
        id
        ...ClaimView_viewer
      }
    }
  """)
  object claimViewQuery extends TaggedNode

  //
  @gql("""
    fragment ClaimView_viewer on Viewer {
      claim(claimId: $claimId) {
        sessionId summary
        timeUpdatedMs timeCreatedMs
        status
        details {
          messages(first: $first) @connection(key: "ClaimView_messages") {
            edges {
              node {
                id
                be {
                  __typename
                  ... on UserUttered {
                    msg
                    when
                  }
                  ... on BotUttered {
                    msg
                    when
                  }
                }
              }
              cursor
            }
          }
        }
      }
      ...AdminCore_viewer
    }

  """)
  object claimViewFrag extends TaggedNode

  val C = ScalaComponent
    .builder[Props]("ClaimView")
    .stateless
    .renderBackend[Backend]
    .buildRelay[relay.generated.ClaimView]

  val Container = FragmentContainer.buildRelay(C, "viewer" -> claimViewFrag)

  class Backend(val $ : BackendScope[Props, Unit]) extends RelayBackend[relay.generated.ClaimView] {
    private val queue = PublishSubject[ChatGrammar]()
    val cancelable    = MultiAssignmentCancelable()

    def render(p: Props) = {

      dom.console.log(relayProps.viewer.claim)
      AdminCore.Container.rprops(relayProps.asInstanceOf[BaseViewerFrag])(
        AdminCore.Props(fluid = true, ctl = p.rc, p = p.page))(
        BreadcrumbDisplay[Page](Home :: ClaimListPage :: p.page :: Nil, rc = p.rc, display = _.title),
        Row(`type` = "flex", gutter = 16)(
          Col(colOps)(Sectional(new Card.Opts {
            override val noHovering = true
            override val title =
              <.span(<.span("Claim"), <.span(" - "), <.code(<.strong(p.page.claimId))).rawElement
          })( /*ChatInterface.C(ChatInterface.Props(queue, msgs, onSend, onChange, s.msg))*/ )),
          Col(colOps)(Sectional(new Card.Opts {
            override val noHovering = true
            override val title      = "Details"
          })())))
    }
  }

  def apply(cp: ClaimPage, r: RouterCtl[Page]) = {
    Routes.baseViewer[relay.generated.ClaimView](
      claimViewQuery,
      r,
      rp => Container.rprops(rp)(Props(r, cp)),
      Some(new js.Object {
        val claimId: String = cp.claimId
        val first: Int      = 1000
      }))
  }

  val colOps: Col.Opt = new Col.Opt {
    override val span = 12
    override val lg   = 12
    override val md   = 24
    override val sm   = 24
    override val xs   = 24
  }
}
