package web.admin.cps

import ant.cps.Layout.{Content, Header, Sider}
import ant.cps.{Icon, Layout, Menu, PopConfirm}
import dispalt.relay.gql
import japgolly.scalajs.react._
import japgolly.scalajs.react.extra.router.RouterCtl
import japgolly.scalajs.react.vdom.html_<^._
import org.scalajs.dom
import relay.components._
import relay.generated.AdminCore_viewer
import relay.generated.lifted.LiftedViewer
import relay.graphql.TaggedNode
import web.admin.data.BaseViewerFrag
import web.admin.mutation.LogoutMutation
import web.admin.routes.{ClaimListPage, Home, Page}
import web.base.all._
import web.cps.{PageFooter, TopSearch}
import web.nav.{NavI, NavItem, SubNavItem}
import web.react.RelayBackend
import web.views.coreLo

import scala.scalajs.js
import scala.scalajs.js.JSConverters._

object AdminCore {
  case class State(collapse: Boolean)

  val Nav: Seq[NavI[Page]] =
    Seq(NavItem("Home", "home", "home", Home), NavItem("Claims", "claims", "code-o", ClaimListPage))

  class Props(val fluid: Boolean, val ctl: RouterCtl[Page], val p: Option[Page])

  object Props {
    def apply(fluid: Boolean, ctl: RouterCtl[Page], p: Page): Props = new Props(fluid, ctl, Some(p))
    def apply(fluid: Boolean, ctl: RouterCtl[Page]): Props          = new Props(fluid, ctl, None)
  }

  class Backend(val $ : BackendScope[Props, Unit]) extends RelayBackend[LiftedViewer[AdminCore_viewer]] {

    lazy val navMap: Map[String, NavI[Page]] = NavI.traverse(Nav, Map.empty)

    val onSiderSelect: Menu.SelectMenuCallback => Unit = { (select: Menu.SelectMenuCallback) =>
      navMap.get(select.key).foreach {
        case nav: NavItem[Page]  => $.props.flatMap(_.ctl.set(nav.page)).runNow()
        case _: SubNavItem[Page] => dom.console.log("Clicked a subnav")
      }
    }

    def logoutConfirm = Callback {
      LogoutMutation.commit(relayProps.viewer.id, relay.environment).foreach {
        case Left(err) => dom.console.error("failed to logout", err.originalError)
        case Right(f)  => dom.console.log(f)
      }
      ()
    }

    val onTopSelect: Menu.SelectMenuCallback => Unit = { (select: Menu.SelectMenuCallback) =>
      select.key match {
        case "logout" =>
        case _        => dom.console.log("unbound action")
      }
    }

    def render(p: Props, pc: PropsChildren) = {
      val (defaultOpen, defaultSelected) = navMap.values.collectFirst {
        case k: NavItem[Page] if p.p.isDefined && k.page == p.p.get =>
          val res: js.Array[String] = {
            val split = k.key.split("[.]", 2)

            if (split.size == 2) js.Array(split(0))
            else js.Array()
          }

          (res, k.key)
      }.unzip

      val usr = Option(relayProps.viewer.user)
      val name = (for {
        fn <- usr.map(_.user.firstName)
        ln <- usr.map(_.user.lastName)
      } yield s"$fn $ln").getOrElse("")

      val containerClass = if (p.fluid) coreLo("fluid") else coreLo("fixed")

      Layout(new Layout.Opts {}.ex(^.minHeight := "100vh"))(
        Sider(new Sider.Opts {
          override val breakpoint     = "lg"
          override val collapsedWidth = 0
        })(
          <.div(^.className := "logo")(<.img(^.src := "/assets/images/logo-mark-color-32x.png")),
          Menu(
            theme               = "dark",
            mode                = "inline",
            defaultSelectedKeys = defaultSelected.toJSArray,
            defaultOpenKeys     = defaultOpen.flatten.toJSArray,
            onSelect            = onSiderSelect)(Nav.toVdomArray(NavI.createNav))),
        Layout(
          Header((new Header.Opts {})(^.background := "#fff", ^.padding := "0"))(
            <.div(^.id := "search-box")(TopSearch.C()),
            <.div(^.id := "profile")(Icon("smile-o"), <.span(name)),
            <.div(^.className := coreLo("header"))(Menu(mode = "horizontal", id = "nav", onSelect = onTopSelect)(
              Menu.Item(key = "logout")(PopConfirm(new PopConfirm.Opt {
                override val title     = js.defined("Confirm you'd like to logout.")
                override val placement = "bottomRight"
                override val onConfirm = js.defined(logoutConfirm.toJsFn)
              })(<.div(Icon("logout"), "Logout")))))),
          Content(new Content.Opts {}.ex(^.overflow := "initial"))(<.div(^.className := containerClass)(pc)),
          PageFooter()))
    }
  }

  @gql("""
      fragment AdminCore_viewer on Viewer {
        id
        user {
          user {
            id
            firstName @scalajs(extends: "Foo")
            lastName
         }
         roles
         permissions
        }
      }
    """)
  object localViewer extends TaggedNode

  val C = ScalaComponent
    .builder[Props](toString)
    .stateless
    .renderBackendWithChildren[Backend]
    .buildRelay[LiftedViewer[AdminCore_viewer]]

  val Container = FragmentContainer.buildRelay(C, "viewer" -> localViewer)

  override def toString = "AdminCore"
}
